importPackage(java.lang);
importPackage(java.util);
importPackage(java.io);
importPackage(Packages.org.apache.commons.httpclient);
importPackage(Packages.org.apache.commons.httpclient.methods);
importPackage(Packages.org.apache.commons.httpclient.auth);
importClass(Packages.com.alarmpoint.integrationagent.security.EncryptionUtils);


load("integrationservices/notifications2csv/json2.js");
load("integrationservices/notifications2csv/config.js");

var PWD = getPassword( PWD_FILE );

/**
 * This method is intended to be invoked by the command line using APClient.bin.  It takes two parameters:
 *
 * range - date or date interval in the ISO 8601 format.  
 * status - a notification status category - one of ALL, PENDING, DELIVERED, RESPONDED or FAILED
 * 
 * The following sample command line searches for notifications created between March 11th 2013 and March 13th 2013 of any status:
 *
 * APClient.bin --map-data notifications2csv "2013-03-11T00:00:00Z/2013-03-13T00:00:00Z" "ALL"
 *
 * Note that range can be a duration that represents the amount of time before "now" (actually
 * 5 minutes ago). This is 1 month ago:
 *
 * APClient.bin --map-data notifications2csv "P1M" "ALL"
 */
function apia_input(apxml)
{
  // collect the parameters and compose the URL to call
  var range = apxml.getValue("range");
  var status = apxml.getValue("status");

  // The ISO specification requires two entries for a duration.
  // However, when scheduling this, we'd just like to take "now". 
  // So the javascript calculates "now" and appends it to the parm.
  // I found that my machine seems to be 1 minute faster than the
  // server, so we just subtract 5 minutes from "now" and use that
  // date/time. 
  if( range.indexOf( "/" ) == -1 )
    range = range + "/" + toISOString(new Date(new Date().setMinutes(new Date().getMinutes()-5)));
   
  var url = xMOD + "?status=" + status + "&range=" + range;

  ServiceAPI.getLogger().debug( "URL: " + url );

  // get the first page of notifications
  var response = getPageOfNotifications(url);

  // If an error was encountered, it was printed and
  // so we exit gracefully (ish).
  if( response == null )
    return;

  ServiceAPI.getLogger().info( "Start of notifications." );

  // open a file to spool the notifications to
  var fstream = new FileWriter( OUTPUT_PATH + "notifications_" + Date.now() + ".csv");
  var file = new BufferedWriter(fstream);
  file.write( getHeaders() );
  file.newLine();

  ServiceAPI.getLogger().info("Total notifications to process: " + 
        (response == null ? 0 : response.total) );

  // if the response was null (errored) or there are no further 
  // notifications, we're done
  if(response == null || response.notifications.length == 0)
  {
    ServiceAPI.getLogger().info("End of notifications reached.");
    file.close();
    return;
  }    

  var notifications = response.notifications;

  // iterate through the remaining pages
  outputNotifications(notifications, file);
  while(response.nextRecordsUrl != null && response.nextRecordsUrl != "")
  {
    response = getPageOfNotifications(response.nextRecordsUrl);
    outputNotifications(response.notifications, file);
  }

  file.close();
  ServiceAPI.getLogger().info("End of notifications reached.");
}

/**
 * Write a CSV line to the file for each notification in the page.
 */
function outputNotifications(notifications, file)
{
  for(var i = 0; i < notifications.length; i++) 
  {
    file.write(notification2csv(notifications[i])); 
    file.newLine();
  }
}

/**
 * Perform an HTTP GET to retrieve a page of notifications.
 */
function getPageOfNotifications(url)
{
  var client = new HttpClient();
  client.getParams().setParameter("content-encoding", "charset=UTF-8");
  client.getParams().setAuthenticationPreemptive(true);

  var method = new GetMethod(url);
  method.addRequestHeader("content-type", "application/json");

  var creds = new UsernamePasswordCredentials( USER, PWD );
  client.getState().setCredentials(AuthScope.ANY, creds);

  var responseCode = client.executeMethod(method);
 
  ServiceAPI.getLogger().debug("Recieved " + responseCode + " from server.");

  var responseBody = method.getResponseBodyAsString();
  var responseObj  = JSON.parse( responseBody );
  ServiceAPI.getLogger().debug( "Response(string): " + responseBody );

  // Catch any errors
  if( responseObj.type != undefined ) {
    ServiceAPI.getLogger().error( responseObj.type +": " + responseObj.message + "." );
    return null;

  }

  ServiceAPI.getLogger().debug( "total: " + responseObj.total );
  
  return responseObj;
}

/**
 * Returns a string suitable for a header row in a CSV file.
 *
 * This loops through the header entries in the COLUMNS variable
 * and adds them to the header line
 */ 
function getHeaders()
{
  var temp = new Array();
  for( header in COLUMNS ) 
    temp.push( header );
  return temp.join( SEP_CHAR );

}

/**
 * Returns a CSV representation of JSON notification.
 *
 * This loops through the entries in COLUMNS and evals
 * each to get the corresponding value. 
 */
function notification2csv(notification)
{

  var temp = new Array();
  for( col in COLUMNS ) 
   temp.push( eval( COLUMNS[ col ] ) );

  return temp.join( SEP_CHAR );

}




/**
 * Decrypts the password for the ServiceDeskUser used by the integration
 * @param passwordFile file/path of the file containing the encrypted password
 * @return decrypted password or an empty string if the password cannot be decrypted
 */
function getPassword(passwordFile)
{
  try
  {
    var encryptionUtils = new EncryptionUtils();
    var file = new File(passwordFile);
    return encryptionUtils.decrypt(file);
  }
  catch (e)
  {
    return "";
  }
}


/**
 * Returns an ISO 8601 compliant date string. 
 * @parm date Date object
 * @return a string in ISO 8601 format: 2013-12-01T12:14Z
 */
function toISOString( date )  {

    function pad(number) {
      if ( number < 10 ) {
        return '0' + number;
      }
      return number;
    }

    return date.getUTCFullYear() +
      '-' + pad( date.getUTCMonth() + 1 ) +
      '-' + pad( date.getUTCDate() ) +
      'T' + pad( date.getUTCHours() ) +
      ':' + pad( date.getUTCMinutes() ) +
      ':' + pad( date.getUTCSeconds() ) +
      '.' + (date.getUTCMilliseconds() / 1000).toFixed(3).slice( 2, 5 ) +
      'Z';
    
}

